# sample_three.py

"""

Author  : Werner F. Bruhin <wernerfbd@gmx.ch>
          Modified and updated by Ecco
Purpose : How to I18N enable an application
Created : 04/06/2012 (d/m/y)
Updated : 11/09/2019 (d/m/y)

Inspired by the I18N wxPython demo and the  
Internationalization page on the wxPython wiki.

"""

import os
import sys
import wx
import wx.lib.sized_controls as sc
from wx.lib.mixins.inspection import InspectionMixin
import wx.adv

# def _displayHook
# class MyFrame
# class MyApp

#---------------------------------------------------------------------------

# Install a custom displayhook to keep Python from setting the global
# _ (underscore) to the value of the last evaluated expression.
# If we don't do this, our mapping of _ to gettext can get overwritten.
# This is useful/needed in interactive debugging with PyShell.
def _displayHook(obj):
    """
    Custom display hook to prevent Python stealing '_'.
    """

    if obj is not None:
        print(repr(obj))

# Add translation macro to builtin similar to what gettext does.
import builtins
builtins.__dict__['_'] = wx.GetTranslation

#---------------------------------------------------------------------------

class MyFrame(sc.SizedFrame):
    """
    The main frame for the i18n demo.
    """
    def __init__(self, parent, **kwds):
        """
        A sample application to demonstrate how to enable I18N support.
        """
        super(MyFrame, self).__init__(parent, **kwds)

        #------------

        self.appName = "I18N sample application"

        #------------
        
        # Configuration folder.
        sp = wx.StandardPaths.Get()
        self.configLoc = sp.GetUserConfigDir()
        self.configLoc = os.path.join(self.configLoc, self.appName)
        # Win: C:\Users\userid\AppData\Roaming\appName
        # Nix: \home\userid\appName

        # AppConfig stuff is here.
        self.appConfig = wx.FileConfig(appName=self.appName,
                                       vendorName="Who you wish",
                                       localFilename=os.path.join(self.configLoc,
                                                                  "AppConfig"))

        #------------
        
        # Simplified init method.
        self.SetProperties()
        self.CreateMenu()
        self.CreateCtrls()
        self.BindEvents()
        self.DoLayout()

    #-----------------------------------------------------------------------

    def SetProperties(self):
        """
        Set properties for my frame.
        """

        self.SetTitle(_("The I18N sample application"))
        self.SetIcon(wx.Icon('icons/wxwin.ico'))        
        self.SetSize((440, 240))
        self.SetMinSize((440, 240))


    def CreateMenu(self):
        """
        Make menu for my frame.
        """

        menubar = wx.MenuBar()
                
        # File menu.
        fileMenu = wx.Menu()
        closeMenuItem = fileMenu.Append(-1,
                                        _("Close"),
                                        _("Close the application"))
        self.Bind(wx.EVT_MENU, self.OnClose, closeMenuItem)
        menubar.Append(fileMenu, _("&File"))
        
        # Edit menu.
        manageMenu = wx.Menu()
        manageSomethingMenuItem = manageMenu.Append(-1,
                                            _("Edit something"),
                                            _("Edit an entry of something"))
        self.Bind(wx.EVT_MENU, self.OnEditSomething, manageSomethingMenuItem)
        
        menubar.Append(manageMenu, _("&Edit"))
        
        # Help menu.
        helpMenu = wx.Menu()
        aboutMenuItem = helpMenu.Append(-1,
                                        _("&About"),
                                        _("About the program"))
        self.Bind(wx.EVT_MENU, self.OnAboutBox, aboutMenuItem)
        menubar.Append(helpMenu, _("&Help"))
        
        self.SetMenuBar(menubar)

       
    def CreateCtrls(self):
        """
        ...
        """
        
        pane = self.GetContentsPane()        

        #------------
        
        cPane = sc.SizedPanel(pane)        
        cPane.SetSizerType("grid", options={"cols": 2})
        cPane.SetSizerProps(align="center")

        #------------
        
        self.st = wx.StaticText(cPane, wx.ID_ANY, _("A nice label for the TextCtrl"))

        self.tc = wx.TextCtrl(cPane, wx.ID_ANY, size=(220, -1))
         
        self.searchSt = wx.StaticText(cPane, wx.ID_ANY, _("a search control"))

        self.searchC = wx.SearchCtrl(cPane, wx.ID_ANY, size=(220, -1))
        
        self.sline = wx.StaticLine(pane, wx.ID_ANY)

        self.btn = wx.Button(pane, wx.ID_ANY, _("Open a file dialog"))

        self.langLbl = wx.StaticText(pane, label=_("Language :"))
        
        #------------

        langList = ["English", "Deutsche", "French"]
        langList.sort()
        
        self.cbLang = wx.ComboBox(pane,
                                  choices=langList,
                                  style=wx.CB_READONLY,
                                  size=(240, -1))
        # Read the value from key "Language".
        oldLang = self.appConfig.Read(key="Language")        
        print("Current program language : %s" % oldLang)
        self.cbLang.SetValue(oldLang)
        self.cbLang.Layout()

        #------------
        
        self.infoLbl = wx.StaticText(pane, label=_("Quit and restart program."))
        self.infoLbl.SetForegroundColour("red")
        

    def BindEvents(self):
        """
        Bind all the events related to my frame.
        """

        self.btn.Bind(wx.EVT_BUTTON, self.OnButton)
        self.Bind(wx.EVT_COMBOBOX, self.OnComboBox, self.cbLang)
        

    def DoLayout(self):
        """
        ...
        """

        self.st.SetSizerProps(valign="center")
        self.searchSt.SetSizerProps(valign="center")
        self.sline.SetSizerProps(expand=True)
        self.btn.SetSizerProps(align="center")
        self.cbLang.SetSizerProps(align="center")
        self.infoLbl.SetSizerProps(align="center")
        self.langLbl.SetSizerProps(align="center")

        # A little trick to make sure that you can't resize the 
        # dialog to less screen space than the controls need.        
        self.Fit()

        
    def OnComboBox(self, event):
        """
        ComboBox handler.
        """
        
        newLang = self.cbLang.GetValue()
        print("New program language : %s" % newLang)

        # Set the value to "Language".
        self.appConfig.Write(key="Language", value=self.cbLang.GetValue())            
        self.appConfig.Flush()

        
    def OnButton(self, event):
        """
        Button handler.
        """
        
        wildcard = "Python source (*.py)|*.py|"     \
                   "Compiled Python (*.pyc)|*.pyc|" \
                   "SPAM files (*.spam)|*.spam|"    \
                   "Egg file (*.egg)|*.egg|"        \
                   "All files (*.*)|*.*"
        
        with wx.FileDialog(
            self, message=_("Choose a file"),
            defaultDir=os.getcwd(), 
            defaultFile="",
            wildcard=wildcard,
            style=wx.FD_OPEN | wx.FD_MULTIPLE | wx.FD_CHANGE_DIR
            ) as dlg:

            # Show the dialog and retrieve the user response. 
            # If it is the OK response, process the data.
            if dlg.ShowModal() == wx.ID_OK:
                # This returns a Python list of files that were selected.
                paths = dlg.GetPaths()

        
    def OnClose(self, event):
        """
        Button handler.
        """
        
        self.Destroy()
        event.Skip()

        
    def OnEditSomething(self, event):
        """
        ...
        """
        
        dlg = wx.MessageDialog(self, _("Hello from Python and wxPython !"),
                               _("Edit"),
                               wx.OK | wx.ICON_INFORMATION
                               )
        dlg.ShowModal()
        dlg.Destroy()

        
    def OnAboutBox(self, event):
        """
        ...
        """
        
        dlg = wx.MessageDialog(self, _("Hello from Python and wxPython !"),
                               _("About"),
                               wx.OK | wx.ICON_INFORMATION
                               )
        dlg.ShowModal()
        dlg.Destroy()

#---------------------------------------------------------------------------

class MyApp(wx.App, InspectionMixin):
    """
    The Application, using WIT to help debugging.
    """
    def OnInit(self):
        """
        Do application initialization work, e.g. define application globals.
        """

        # Name for my application.
        self.appName = "I18N sample application"
  
        #------------
        
        # Simplified init method.
        self.DoConfig()
        self.Init() # InspectionMixin
        # work around for Python stealing "_".
        sys.displayhook = _displayHook
        
        #------------        
        
        # Set language stuff and update to last used language.        
        self.locale = None
        wx.Locale.AddCatalogLookupPathPrefix("locale")
        self.UpdateLanguage(self.appConfig.Read("Language"))

        return True

    #-----------------------------------------------------------------------
    
    def DoConfig(self):
        """
        Setup an application configuration file.
        """

        # Configuration folder.
        sp = wx.StandardPaths.Get()
        self.configLoc = sp.GetUserConfigDir()
        self.configLoc = os.path.join(self.configLoc, self.appName)
        # Win : C:\Users\userid\AppData\Roaming\appName
        # Nix : \home\userid\appName

        if not os.path.exists(self.configLoc):
            os.mkdir(self.configLoc)

        # AppConfig stuff is here.
        self.appConfig = wx.FileConfig(appName=self.appName,
                                       vendorName="Who you wish",
                                       localFilename=os.path.join(self.configLoc,
                                                                  "AppConfig"))
    
        if not self.appConfig.HasEntry("Language"):
            # On first run we default to English.
            self.appConfig.Write(key="Language", value="English")
            
        self.appConfig.Flush()


    def UpdateLanguage(self, lang):
        """
        Update the language to the requested one.
        
        Make *sure* any existing locale is deleted before the new
        one is created. The old C++ object needs to be deleted
        before the new one is created, and if we just assign a new
        instance to the old Python variable, the old C++ locale will
        not be destroyed soon enough, likely causing a crash.
        
        :param string `lang`: one of the supported language codes.        
        """

        # Language domain.
        langDomain = "I18Nwxapp"
        
        # Languages you want to support.
        supLang = {"English": wx.LANGUAGE_ENGLISH,
                   "Deutsche": wx.LANGUAGE_GERMAN,
                   "French": wx.LANGUAGE_FRENCH,
                  }
                   
        # If an unsupported language is requested default to English.
        if lang in supLang:
            selLang = supLang[lang] 
        else:
            selLang = wx.LANGUAGE_ENGLISH
            
        if self.locale:
            assert sys.getrefcount(self.locale) <= 2
            del self.locale
        
        # Create a locale object for this language.
        self.locale = wx.Locale(selLang)
        if self.locale.IsOk():
            self.locale.AddCatalog(langDomain)
        else:
            self.locale = None
        
#---------------------------------------------------------------------------
        
if __name__ == "__main__":
    app = MyApp(redirect=False)
    frame = MyFrame(None)
    frame.Show()
    app.MainLoop()
