# sample_two.py

"""

Author  : Werner F Bruhin <wernerfbd@gmx.ch>
          Modified and updated by Ecco
Purpose : Demonstrating i18n
Created : 15/04/2014 (d/m/y)
Updated : 11/09/2019 (d/m/y)
  
"""

import os
import sys
import configparser
import wx
import wx.lib.sized_controls as SC
import wx.lib.mixins.inspection as WIT

_ = wx.GetTranslation

# def _displayHook
# class MyDialog
# class MyFrame
# class MyApp

#---------------------------------------------------------------------------

# Install a custom displayhook to keep Python from setting the global
# _ (underscore) to the value of the last evaluated expression. 
# If we don't do this, our mapping of _ to gettext can get overwritten.
# This is useful/needed in interactive debugging with PyShell.
def _displayHook(obj):
    """
    Custom display hook to prevent Python stealing '_'.
    """
    
    if obj is not None:
        print(repr(obj))


if hasattr(sys, "frozen"):
    basedir = os.getcwd()
else:
    basedir, f = os.path.split(__file__)

#---------------------------------------------------------------------------

class MyDialog(SC.SizedDialog):
    """
    The dialog for the i18n demo.
    """
    def __init__(self, parent, id):
        super(MyDialog, self).__init__(None, -1,
                                       _("SizedForm MyDialog"),
                                       style=wx.DEFAULT_DIALOG_STYLE |
                                             wx.RESIZE_BORDER)

        #------------
        
        # Simplified init method.
        self.SetProperties()
        self.CreateCtrls()
        self.DoLayout()

    #-----------------------------------------------------------------------
        
    def SetProperties(self):
        """
        Set properties for my dialog.
        """

        self.SetMinSize((255, 255))


    def CreateCtrls(self):
        """
        ...
        """

        pane = self.GetContentsPane()
        pane.SetSizerType("form")

        #------------

        # Row 1.
        wx.StaticText(pane, -1, _("Name"))
        self.textCtrl = wx.TextCtrl(pane, -1, _("Your name here"))

        # Row 2.
        wx.StaticText(pane, -1, _("Email"))
        self.emailCtrl = wx.TextCtrl(pane, -1, "")

        # Row 3.
        wx.StaticText(pane, -1, _("Gender"))
        wx.Choice(pane, -1, size=(100, -1), choices=["male", "female"])

        # Row 4.
        wx.StaticText(pane, -1, _("State"))
        wx.TextCtrl(pane, -1, size=(100, -1)) # Two chars for state.

        # Row 5.
        wx.StaticText(pane, -1, _("Title"))        

        # Here's how to add a "nested sizer using sized_controls.
        self.radioPane = SC.SizedPanel(pane, -1)

        # Make these children of the radioPane to  
        # have them use the horizontal layout.
        wx.RadioButton(self.radioPane, -1, _("Mr."))
        wx.RadioButton(self.radioPane, -1, _("Mrs."))
        wx.RadioButton(self.radioPane, -1, _("Dr."))
        # End row 5.

        # Add dialog buttons.
        self.SetButtonSizer(self.CreateStdDialogButtonSizer(wx.OK | wx.CANCEL))

        
    def DoLayout(self):
        """
        ...
        """

        self.textCtrl.SetSizerProps(expand=True)
        self.emailCtrl.SetSizerProps(expand=True)
        self.radioPane.SetSizerType("horizontal")
        self.radioPane.SetSizerProps(expand=True)

        # A little trick to make sure that you can't resize the 
        # dialog to less screen space than the controls need.
        self.Fit()
        
#---------------------------------------------------------------------------

class MyFrame(SC.SizedFrame):
    """
    The main frame for the i18n demo.
    """
    def __init__(self, *args, **kwargs):
        """
        Constructor.
        """
        super(MyFrame, self).__init__(*args, **kwargs)

        #------------

        # Return ini file. 
        self.cf = os.path.join(basedir, "i18nconfig.ini")
        # Get a config object.
        self.config = configparser.ConfigParser()
        # Read the file "i18nconfig.ini". 
        self.config.read(self.cf)
        
        #------------
        
        # Simplified init method.
        self.SetProperties()
        self.CreateCtrls()
        self.BindEvents()
        self.DoLayout()
        
    #-----------------------------------------------------------------------
        
    def SetProperties(self):
        """
        Set properties for my frame.
        """

        self.SetIcon(wx.Icon('icons/wxwin.ico'))
        self.SetMinSize((320, 170))
        

    def CreateCtrls(self):
        """
        ...
        """

        sp = self.GetContentsPane()

        #------------
        
        self.niceLbl = wx.StaticText(sp, wx.ID_ANY, _("Some nice text to translate"))       
        self.btn = wx.Button(sp, wx.ID_ANY, _("Click to start a dialog"))
        self.langLbl = wx.StaticText(sp, label=_("Language :"))
        
        #------------
        
        langList = ["English", "Deutsche", "French"]
        langList.sort()
        
        self.cbLang = wx.ComboBox(sp,
                                  choices=langList,
                                  style=wx.CB_READONLY,
                                  size=(240, -1))
        # Read the value from section "Settings", option "language".
        oldLang = self.config.get("Settings", "language")
        print("Current program language : %s" % oldLang)
        self.cbLang.SetValue(oldLang)
        self.cbLang.Layout()

        #------------
        
        self.infoLbl = wx.StaticText(sp, label=_("Quit and restart program."))
        self.infoLbl.SetForegroundColour("red")
        

    def BindEvents(self):
        """
        Bind all the events related to my frame.
        """

        self.btn.Bind(wx.EVT_BUTTON, self.OnButton)
        self.Bind(wx.EVT_COMBOBOX, self.OnComboBox, self.cbLang)
        

    def DoLayout(self):
        """
        ...
        """

        self.niceLbl.SetSizerProps(align="center")
        self.btn.SetSizerProps(align="center")
        self.langLbl.SetSizerProps(align="center")
        self.cbLang.SetSizerProps(align="center")
        self.infoLbl.SetSizerProps(align="center")
        
        self.Fit()

        
    def OnComboBox(self, event):
        """
        ComboBox handler.
        """
        
        newLang = self.cbLang.GetValue()
        print("New program language : %s" % newLang)

        # Set the value to "language".
        self.config.set("Settings", "language", value=self.cbLang.GetValue())

        # Rewrite the configuration to the ".ini" file.
        with open(self.cf, "w") as config_file:
            self.config.write(config_file)
            
         
    def OnButton(self, event):
        """
        Button handler.
        """

        with MyDialog(self, wx.ID_ANY) as dlg:
            dlg.ShowModal()

#---------------------------------------------------------------------------
            
class MyApp(WIT.InspectableApp):
    """
    The Application, using WIT to help debugging.
    """
    def OnInit(self):
        """
        Do application initialization work, e.g. define application globals.
        """

        # Simplified init method.
        self.InitConfig()
        self.Init() # InspectionMixin
        # work around for Python stealing "_".
        sys.displayhook = _displayHook

        #------------

        # Return locale folder.
        localeDir = os.path.join(basedir, "locale")
        
        # Set language stuff and update to last used language.
        self.locale = None
        wx.Locale.AddCatalogLookupPathPrefix(localeDir)
        self.UpdateLanguage(self.appConfig.get("Settings", "language"))

        return True

    #-----------------------------------------------------------------------
    
    def InitConfig(self):
        """
        Initialize a configuration file if not present.
        """
        
        cf = os.path.join(basedir, "i18nconfig.ini")

        if not os.path.exists(cf):
            config = configparser.ConfigParser()
            config.add_section("Settings")
            # Set the default value to "language".
            config.set("Settings", "language", "English")
            # Write the configuration to the ".ini" file.
            with open(cf, "w") as config_file:
                config.write(config_file)       
      
        self.appConfig = configparser.ConfigParser()
        self.appConfig.read(cf)

    
    def UpdateLanguage(self, lang):
        """
        Update language locale.
        """

        # The key has to match the folder name, so you
        # could also use 'de_de', 'de_ch' etc if needed.
        supportedLangs = {"English": wx.LANGUAGE_ENGLISH,
                          "Deutsche": wx.LANGUAGE_GERMAN,
                          "French": wx.LANGUAGE_FRENCH,
                         }
            
        # Make *sure* any existing locale is deleted before the new
        # one is created. The old C++ object needs to be deleted
        # before the new one is created, and if we just assign a new
        # instance to the old Python variable, the old C++ locale will
        # not be destroyed soon enough, likely causing a crash.
        if self.locale:
            assert sys.getrefcount(self.locale) <= 2
            del self.locale

        # Create a locale object for this language.
        if supportedLangs[lang]:
            self.locale = wx.Locale(supportedLangs[lang])
            if self.locale.IsOk():
                self.locale.AddCatalog("i18nDemo")
            else:
                self.locale = None
        else:
            log.error("Requested language is not supported: %s" % lang)

#---------------------------------------------------------------------------
            
if __name__ == "__main__":
    app = MyApp()
    frm = MyFrame(None, title=_("A nice frame title"))
    frm.Show()
    app.MainLoop()
